@@ -8,4 +8,6 @@ Created on Mon Aug  5 20:46:26 2019
 from . import settings
 from .core import Partial, Solver
 from .cache import multi_cached, mono_cached
-from .utils import *
\ No newline at end of file
+from . import soft_functions
+from .utils import *
+from . import solvers
\ No newline at end of file
