@@ -19,12 +19,15 @@ from . import settings
 sb = fw.SignalBox()
 
 EMIT_VECTORS = 'EMIT_VECTORS'
+EMIT_STATE = 'EMIT_STATE'
 GET_VARS = 'GET_VARS'
 GET_STEP_METHODS = 'GET_STEP_METHODS'
 GET_PARTIALS = 'GET_PARTIALS'
 SET_CACHING = 'SET_CACHING'
 GET_CACHE_CLEARS = 'GET_CACHE_CLEARS'
 
+      
+
 class Partial():
     ''' A base class responsible for a set of variables 
     
@@ -35,6 +38,7 @@ class Partial():
     
     def __init__(self):
         self.npsolve_vars = {}
+        self.state = {}
         self.__cache_methods = self._get_cached_methods()
         self.__cache_clear_functions = self._get_cache_clear_functions()
         self.cache_clear() # Useful for iPython console autoreload.
@@ -51,6 +55,7 @@ class Partial():
         try:
             c = sb.get_container(cid)
             c.get(EMIT_VECTORS, must_exist=True).connect(self.set_vectors)
+            c.get(EMIT_STATE, must_exist=True).connect(self._set_state)
             c.get(GET_VARS, must_exist=True).connect(self._get_vars)
             c.get(GET_STEP_METHODS, must_exist=True).connect(self._get_step_methods)
             c.get(GET_PARTIALS, must_exist=True).connect(self._get_self)
@@ -74,6 +79,17 @@ class Partial():
         '''
         pass
     
+    def _set_state(self, state):
+        """ Set the state dictionary
+        
+        Args:
+            state (dict): The state dictionary
+            
+        Note:
+            The state dictionary de-numpify's scalars by default.
+        """
+        self.state = state
+    
     def _get_vars(self):
         return self.npsolve_vars
 
@@ -95,7 +111,7 @@ class Partial():
         '''
         self.npsolve_vars[name]['init'] = np.atleast_1d(init)
 
-    def add_var(self, name, init, safe=True, **kwargs):
+    def add_var(self, name, init, safe=True, live=True, **kwargs):
         ''' Add a new variable 
         
         Args:
@@ -106,9 +122,11 @@ class Partial():
         '''
         if safe and name in self.npsolve_vars:
             raise KeyError(str(name) + ' already exists')
-        self.npsolve_vars[name] = {}
-        self.set_init(name, init)
-        self.set_meta(name, **kwargs)
+        self.state[name] = np.atleast_1d(init)
+        if live:
+            self.npsolve_vars[name] = {}
+            self.set_init(name, init)
+            self.set_meta(name, **kwargs)
     
     def clear_vars(self):
         self.npsolve_vars = {}
@@ -177,6 +195,7 @@ class Solver():
         self._cache_clear_functions = []
         self.npsolve_isolate = None
         self._container = None
+        self.state = {}
         if settings.AUTO_CONNECT:
             self.setup_signals()
         
@@ -187,8 +206,8 @@ class Solver():
             int: The container id for the signals.
         '''
         self._container = sb.add(activate=True, remove_with=self)
-        signals = [EMIT_VECTORS, GET_VARS, GET_STEP_METHODS, GET_PARTIALS,
-                   SET_CACHING, GET_CACHE_CLEARS]
+        signals = [EMIT_VECTORS, EMIT_STATE, GET_VARS, GET_STEP_METHODS,
+                   GET_PARTIALS, SET_CACHING, GET_CACHE_CLEARS]
         self._signals = {name: sb.get(name) for name in signals}
         return self._container.id
     
@@ -281,6 +300,10 @@ class Solver():
                 state_dct=self.npsolve_state_dct,
                 ret_dct=self.npsolve_ret_dct)
 
+    def _emit_state(self):
+        ''' Pass out vectors and slices to connected Partial instances '''
+        self._signals[EMIT_STATE].emit(state=self.npsolve_state_dct)
+
     def freeze(self):
         ''' Give static copies of vectors to connected Partial instances 
         
@@ -353,13 +376,18 @@ class Solver():
         self.npsolve_state_dct = state_dct
         self.npsolve_ret_dct = ret_dct
         self._emit_vectors()
+        self._emit_state()
         self._step_methods = self._fetch_step_methods()
         self._cache_clear_functions = self._fetch_cache_clears()
         self._signals[SET_CACHING].emit(enable=True)
 
+    def npsolve_finish(self):
+        """ Tidy up after a round of solving """
+        self._signals[SET_CACHING].emit(enable=False)
+
     @contextmanager
     def pinned(self, dct):
-        ''' A context manager that unpinned all variables on exit '''
+        ''' A context manager that unpins all variables on exit '''
         self.npsolve_init(pinned=dct)
         yield
         state = self.npsolve_state.copy()
@@ -460,14 +488,19 @@ class Solver():
         ''' Split out solution array into dictionary of values 
         
         Args:
-            sol (2D ndarray): An array where columns correspond to state values
+            sol (ndarray): A 1D or 2D array where columns correspond to state
+                values
         
         This convenience method splits out a 2D array into a dictionary of
         vectors or arrays, with variables as keys.
         '''
         d = {}
-        for key, slc in self.npsolve_slices.items():
-            d[key] = sol[:,slc]
+        if sol.ndim == 1:
+            for key, slc in self.npsolve_slices.items():
+                d[key] = sol[slc]
+        if sol.ndim == 2:
+            for key, slc in self.npsolve_slices.items():
+                d[key] = sol[:,slc]
         return d
     
     def get_state_dct(self, squeeze=True, unitise=True):
@@ -482,3 +515,25 @@ class Solver():
                     dct[k] = v.item()
         return dct
     
+    def connect(self, partials):
+        """ Connect a dict or list of partials to the Solver instance 
+        
+        Args:
+            partials (list, dict, Partial): A list or dictionary of Partial
+                instances.
+                
+        """
+        if not isinstance(partials, (dict, list, Partial)):
+            raise ValueError('partials argument must be a list or dict of ' +
+                             'npsolve.Partial instances.')
+        self.remove_signals()
+        self.setup_signals()
+        if isinstance(partials, dict):
+            for partial in partials.values():
+                partial.connect()
+        elif isinstance(partials, list):
+            for partial in partials:
+                partial.connect()
+        else:
+            partials.connect()
+        self.close_signals()
\ No newline at end of file
